@@ -23,7 +23,6 @@
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
-#include <xcb/xcb.h>
 #include <xkbcommon/xkbcommon-compose.h>
 
 #if GTK_CHECK_VERSION(2, 24, 0)
@@ -428,13 +427,7 @@ static void fcitx_im_context_finalize(GObject *obj) {
     g_clear_pointer(&context->preedit_string, g_free);
     g_clear_pointer(&context->surrounding_text, g_free);
     g_clear_pointer(&context->attrlist, pango_attr_list_unref);
-//    g_queue_clear_full(&context->gdk_events, (GDestroyNotify)gdk_event_free);
-    g_return_if_fail (&context->gdk_events != NULL);
-	
-    if (gdk_event_free != NULL)
-        g_queue_foreach (&context->gdk_events, (GFunc) gdk_event_free, NULL);
-	
-    g_queue_clear (&context->gdk_events);
+    g_queue_clear_full(&context->gdk_events, (GDestroyNotify)gdk_event_free);
 
     G_OBJECT_CLASS(parent_class)->finalize(obj);
 }
