@@ -15,13 +15,16 @@
 
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
-#include <gdk/gdkx.h>
+
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
-#include <xcb/xcb.h>
 #include <xkbcommon/xkbcommon-compose.h>
 
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#endif
+
 #ifdef GDK_WINDOWING_WAYLAND
 #include <gdk/gdkwayland.h>
 #endif
@@ -440,9 +443,11 @@ static void fcitx_im_context_init(FcitxIMContext *context, gpointer) {
     if (context->is_wayland) {
         fcitx_g_client_set_display(context->client, "wayland:");
     } else {
+#ifdef GDK_WINDOWING_X11
         if (GDK_IS_X11_DISPLAY(gdk_display_get_default())) {
             fcitx_g_client_set_display(context->client, "x11:");
         }
+#endif
     }
     g_signal_connect(context->client, "connected",
                      G_CALLBACK(_fcitx_im_context_connect_cb), context);
@@ -487,13 +492,7 @@ static void fcitx_im_context_finalize(GObject *obj) {
     g_clear_pointer(&context->preedit_string, g_free);
     g_clear_pointer(&context->surrounding_text, g_free);
     g_clear_pointer(&context->attrlist, pango_attr_list_unref);
-//    g_queue_clear_full(&context->gdk_events, (GDestroyNotify)gdk_event_free);
-    g_return_if_fail (&context->gdk_events != NULL);
-	
-    if (gdk_event_free != NULL)
-        g_queue_foreach (&context->gdk_events, (GFunc) gdk_event_free, NULL);
-	
-    g_queue_clear (&context->gdk_events);
+    g_queue_clear_full(&context->gdk_events, (GDestroyNotify)gdk_event_free);
 
     G_OBJECT_CLASS(parent_class)->finalize(obj);
 }
@@ -502,21 +501,23 @@ static void fcitx_im_context_finalize(GObject *obj) {
 static void fcitx_im_context_set_client_window(GtkIMContext *context,
                                                GdkWindow *client_window) {
     FcitxIMContext *fcitxcontext = FCITX_IM_CONTEXT(context);
-    if (client_window != fcitxcontext->client_window) {
-        delete fcitxcontext->candidate_window;
-        fcitxcontext->candidate_window = nullptr;
+    if (client_window == fcitxcontext->client_window) {
+        return;
     }
+    delete fcitxcontext->candidate_window;
+    fcitxcontext->candidate_window = nullptr;
     if (!client_window)
         return;
 
     g_clear_object(&fcitxcontext->client_window);
     fcitxcontext->client_window = GDK_WINDOW(g_object_ref(client_window));
-    if (!fcitxcontext->candidate_window) {
-        fcitxcontext->candidate_window = new Gtk3InputWindow(
-            _uiconfig, fcitxcontext->client, fcitxcontext->is_wayland);
-        fcitxcontext->candidate_window->setParent(fcitxcontext->client_window);
-        fcitxcontext->candidate_window->setCursorRect(fcitxcontext->area);
-    }
+
+    _fcitx_im_context_set_capability(fcitxcontext, FALSE);
+
+    fcitxcontext->candidate_window = new Gtk3InputWindow(
+        _uiconfig, fcitxcontext->client, fcitxcontext->is_wayland);
+    fcitxcontext->candidate_window->setParent(fcitxcontext->client_window);
+    fcitxcontext->candidate_window->setCursorRect(fcitxcontext->area);
 }
 
 static gboolean
@@ -1084,7 +1085,10 @@ void _fcitx_im_context_set_capability(FcitxIMContext *fcitxcontext,
         if (fcitxcontext->is_wayland) {
             flags |= (guint64)fcitx::FcitxCapabilityFlag_RelativeRect;
         }
-        flags |= (guint64)fcitx::FcitxCapabilityFlag_ClientSideInputPanel;
+        if (fcitxcontext->client_window != NULL &&
+            gdk_window_is_visible(fcitxcontext->client_window)) {
+            flags |= (guint64)fcitx::FcitxCapabilityFlag_ClientSideInputPanel;
+        }
         flags |= (guint64)fcitx::FcitxCapabilityFlag_KeyEventOrderFix;
         flags |= (guint64)fcitx::FcitxCapabilityFlag_ReportKeyRepeat;
 
@@ -1463,6 +1467,7 @@ static gboolean _key_is_modifier(guint keyval) {
     }
 }
 
+#ifdef GDK_WINDOWING_X11
 void send_uuid_to_x11(Display *xdisplay, const guint8 *uuid) {
     Atom atom = XInternAtom(xdisplay, "_FCITX_SERVER", False);
     if (!atom) {
@@ -1484,9 +1489,11 @@ void send_uuid_to_x11(Display *xdisplay, const guint8 *uuid) {
     XSendEvent(xdisplay, window, False, NoEventMask, &ev);
     XSync(xdisplay, False);
 }
+#endif
 
 void _fcitx_im_context_connect_cb(FcitxGClient *im, void *user_data) {
     FcitxIMContext *context = FCITX_IM_CONTEXT(user_data);
+#ifdef GDK_WINDOWING_X11
     Display *display = NULL;
     if (context->client_window) {
         if (GDK_IS_X11_WINDOW(context->client_window)) {
@@ -1503,6 +1510,7 @@ void _fcitx_im_context_connect_cb(FcitxGClient *im, void *user_data) {
     if (display) {
         send_uuid_to_x11(display, fcitx_g_client_get_uuid(im));
     }
+#endif
 
     _fcitx_im_context_set_capability(context, TRUE);
     if (context->has_focus && _focus_im_context == (GtkIMContext *)context &&
